-- ***** FavoritePet *****
--= IsInteracting(), if they are in combat, or if they have a merchant / banker / fence summoned, then delay or cancel change as neccessary
-- Add in if they are dead, or in battleground or other zone that does not allow pets (and mounts) GetCurrentBattlegroundId()

--------------------------------------------------
-- Initialize addon variables
--------------------------------------------------
FavoritePet = {}
FavoritePet.name = "FavoritePet"
FavoritePet.slashCommand = "/fpet"
FavoritePet.debug = false
FavoritePet.website = "https://www.esoui.com/downloads/info2917-FavoritePet.html"
FavoritePet.version = "1.4.1 (20210711)"

FavoritePet.collectible = 0
ZO_CreateStringId("SI_BINDING_NAME_FAVPET", "Random Favorite Pet")


--------------------------------------------------
-- Link local variables to the in-game Globals
--------------------------------------------------
FavoritePet.typePets = COLLECTIBLE_CATEGORY_TYPE_VANITY_PET -- ZOS GLOBAL
FavoritePet.cooldownError = COLLECTIBLE_USAGE_BLOCK_REASON_ON_COOLDOWN -- ZOS GLOBAL


--------------------------------------------------
-- Default saved variable settings
--------------------------------------------------
FavoritePet.globalDefaults = {
	unlockedPetsList = {},
	unlockedPetsCount = 0
}


FavoritePet.characterDefaults = {
	favoritePetsList = {},
	switchOnCombat = false,
	switchOnZone = false,
	switchOnMount = false,
	switchOnDismount = false
}


--------------------------------------------------
-- Initialize settings, load saved variables and register event triggers.
--------------------------------------------------
function FavoritePet.Initialize()
	FavoritePet.GSV = ZO_SavedVars:NewAccountWide("FavoritePetSavedVariables", 2, nil, FavoritePet.globalDefaults)
	FavoritePet.CSV = ZO_SavedVars:NewCharacterIdSettings("FavoritePetSavedVariables", 1, nil, FavoritePet.characterDefaults)
	
	FavoritePet.Check()
	
	EVENT_MANAGER:RegisterForEvent(FavoritePet.name, EVENT_PLAYER_ACTIVATED, FavoritePet.Check)
	EVENT_MANAGER:RegisterForEvent(FavoritePet.name, EVENT_COLLECTIBLE_NOTIFICATION_NEW, FavoritePet.Check)
	if FavoritePet.CSV.switchOnCombat then EVENT_MANAGER:RegisterForEvent(FavoritePet.name, EVENT_PLAYER_COMBAT_STATE, FavoritePet.CombatTrigger) end
	if FavoritePet.CSV.switchOnZone then EVENT_MANAGER:RegisterForEvent(FavoritePet.name, EVENT_ZONE_CHANGED, FavoritePet.SwitchPet) end
	if (FavoritePet.CSV.switchOnMount or FavoritePet.CSV.switchOnDismount) then EVENT_MANAGER:RegisterForEvent(FavoritePet.name, EVENT_MOUNTED_STATE_CHANGED, FavoritePet.MountTrigger) end
end


--------------------------------------------------
-- Check that the number of pets saved matches the total number unlocked
--------------------------------------------------
function FavoritePet.Check()
	local upets = GetTotalUnlockedCollectiblesByCategoryType(FavoritePet.typePets)
	local spets = FavoritePet.GSV.unlockedPetsCount
	if FavoritePet.debug then spets = 0 d("FavoritePet: Check") end
	if spets ~= upets then FavoritePet.Populate() end
end


--------------------------------------------------
-- Populate the unlocked pets list and save to saved variables
--------------------------------------------------
function FavoritePet.Populate()
	local pets = {}
	local petcounter = 0
	local unlockedpets = GetTotalUnlockedCollectiblesByCategoryType(FavoritePet.typePets)
	if FavoritePet.debug then d("Function Populate") end
	
	if unlockedpets > 0 then
		for count = 1, GetTotalCollectiblesByCategoryType(FavoritePet.typePets) do
			local index = GetCollectibleIdFromType(FavoritePet.typePets, count)
			local p1, p2, p3, p4, p5, p6, p7, p8, p9 = GetCollectibleInfo(index)
			if p5 then
				petcounter = petcounter + 1
				local petNameClean = ZO_CachedStrFormat(SI_UNIT_NAME, p1)
				pets[petcounter] = {index, petNameClean, p3}
			end
		end
		table.sort(pets, function(a,b) return a[2] < b[2] end)
		if FavoritePet.debug then d("UNLOCKED PETS:") d(pets) d("PETS COUNT: "..unlockedpets) end
	
		FavoritePet.GSV.unlockedPetsList = pets
		FavoritePet.GSV.unlockedPetsCount = unlockedpets
	else
		FavoritePet.GSV.unlockedPetsList = {}
		FavoritePet.GSV.unlockedPetsCount = 0
	end
	--[[ 
		GetCollectibleInfo() returns
		p1 = name(string)
		p2 = description(string)
		p3 = path to the item's texture if unlocked
		p4 = path to the item's texture if locked
		p5 = unlocked(boolean)
		p6 = purchasable(boolean)
		p7 = isActive(boolean)
		p8 = Collectible Category Type (number matches the constant used in pre-initialize)
		p9 = hint text for finding the item
	]]--
end


--------------------------------------------------
-- Set if we switch pet when we exit combat - called by the settings menu
--------------------------------------------------
function FavoritePet.SetCombat(toggle)
	if toggle then
		FavoritePet.CSV.switchOnCombat = true
		EVENT_MANAGER:RegisterForEvent(FavoritePet.name, EVENT_PLAYER_COMBAT_STATE, FavoritePet.CombatTrigger)
	else
		FavoritePet.CSV.switchOnCombat = false
		EVENT_MANAGER:UnregisterForEvent(FavoritePet.name, EVENT_PLAYER_COMBAT_STATE)
	end	
end


--------------------------------------------------
-- Set if we switch pet when we change zones - called by the settings menu
--------------------------------------------------
function FavoritePet.SetZone(toggle)
	if toggle then
		FavoritePet.CSV.switchOnZone = true
		EVENT_MANAGER:RegisterForEvent(FavoritePet.name, EVENT_ZONE_CHANGED, FavoritePet.SwitchPet)
	else
		FavoritePet.CSV.switchOnZone = false
		EVENT_MANAGER:UnregisterForEvent(FavoritePet.name, EVENT_ZONE_CHANGED)
	end
end


--------------------------------------------------
-- Set if we switch mount on mounted - called by the settings menu
--------------------------------------------------
function FavoritePet.SetMounted(toggle)
	if toggle then 
		FavoritePet.CSV.switchOnMount = true
		EVENT_MANAGER:RegisterForEvent(FavoritePet.name, EVENT_MOUNTED_STATE_CHANGED, FavoritePet.MountTrigger())
	else
		FavoritePet.CSV.switchOnMount = false
	end
	
	if ((not FavoritePet.CSV.switchOnMount) and (not FavoritePet.CSV.switchOnDismount)) then
		EVENT_MANAGER:UnregisterForEvent(FavoritePet.name, EVENT_MOUNTED_STATE_CHANGED)
	end
end


--------------------------------------------------
-- Set if we switch mount on dismounted - called by the settings menu
--------------------------------------------------
function FavoritePet.SetDismounted(toggle)
	if toggle then
		FavoritePet.CSV.switchOnDismount = true
		EVENT_MANAGER:RegisterForEvent(FavoritePet.name, EVENT_MOUNTED_STATE_CHANGED, FavoritePet.MountTrigger())
	else
		FavoritePet.CSV.switchOnDismount = false
	end
	
	if ((not FavoritePet.CSV.switchOnMount) and (not FavoritePet.CSV.switchOnDismount)) then
		EVENT_MANAGER:UnregisterForEvent(FavoritePet.name, EVENT_MOUNTED_STATE_CHANGED)
	end
end


--------------------------------------------------
-- Switch pet when we exit combat - triggered by EVENT_PLAYER_COMBAT_STATE
--------------------------------------------------
function FavoritePet.CombatTrigger(eventCode, inCombat)
	if (not inCombat) then FavoritePet.SwitchPet() end
end


--------------------------------------------------
-- Switch pet based on mounted state - triggered by EVENT_MOUNTED_STATE_CHANGED
--------------------------------------------------
function FavoritePet.MountTrigger(eventid, mounted)
	if (mounted and FavoritePet.CSV.switchOnMount) then FavoritePet.SwitchPet() end
	if (not mounted and FavoritePet.CSV.switchOnDismount) then FavoritePet.SwitchPet() end
end


--------------------------------------------------
-- Check the results of the EVENT triggered when UseCollectible is called
--------------------------------------------------
function FavoritePet.SwitchResult(eventCode, blockReason, attemptActivation)	
	if (not FavoritePet.SafeToSwitch()) then EVENT_MANAGER:UnregisterForEvent(FavoritePet.name, EVENT_COLLECTIBLE_USE_RESULT) return end
	if blockReason == FavoritePet.cooldownError then
		if FavoritePet.debug then d("COOLDOWN ERROR") end
		local delay = FavoritePet.CalculateDelay()
		zo_callLater(function() UseCollectible(FavoritePet.collectible) end, delay)
	else
		if FavoritePet.debug then d("COOLDOWN SUCCESSFUL") end
		EVENT_MANAGER:UnregisterForEvent(FavoritePet.name, EVENT_COLLECTIBLE_USE_RESULT)
	end
end


--------------------------------------------------
-- Check if it is safe to switch
--------------------------------------------------
function FavoritePet.SafeToSwitch()
	local InCombat = false
	local IsAssistant = false
	local IsBattleground = false
	local IsDead = false
	
	if IsUnitInCombat("player") then InCombat = true end
	-- if GetActiveCollectibleByType(COLLECTIBLE_CATEGORY_TYPE_ASSISTANT) ~= 0 then IsAssistant = true end
	if GetCurrentBattlegroundId() ~= 0 then IsBattleground = true end
	if IsUnitDeadOrReincarnating("player") then IsDead = true end
	
	if (InCombat or IsAssistant or IsBattleground or IsDead) then return false else return true end
end


--------------------------------------------------
-- Calculate how much of a delay before attempting to use pet
--------------------------------------------------
function FavoritePet.CalculateDelay()
	local delay = 1500
	
	if (IsInteracting() or IsInteractionPending()) then delay = 15000 end
	if IsUnitSwimming("player") then delay = 30000 end
	return delay
end


--------------------------------------------------
-- Randomly set a pet as active
--------------------------------------------------
function FavoritePet.SwitchPet()
	local list = FavoritePet.CSV.favoritePetsList
	local active = GetActiveCollectibleByType(FavoritePet.typePets)
	local usepet = active
	
	
	if (not FavoritePet.SafeToSwitch()) then return end
	if #list > 1 then
		while active == usepet do
			usepet = list[math.random(#list)]
		end
		if FavoritePet.debug then d("Pet ID: "..usepet) end
	else
		usepet = list[1]
	end
	FavoritePet.collectible = usepet
	EVENT_MANAGER:RegisterForEvent(FavoritePet.name,  EVENT_COLLECTIBLE_USE_RESULT, FavoritePet.SwitchResult)
	local delay = FavoritePet.CalculateDelay()
	zo_callLater(function() UseCollectible(FavoritePet.collectible) end, delay)
	--  Will unregister for event upon successful usecollectible	
end


--------------------------------------------------
-- Check to see if a pet is in the favorites list - used for menu settings
--------------------------------------------------
function FavoritePet.CheckIfFavorite(num)
	if FavoritePet.debug then d("Check If Favorite") end

	local fav = FavoritePet.CSV.favoritePetsList
	local found = false
	
	if fav then
		for count = 1, #fav do
			if num == fav[count] then found = true break end	
		end
	end
	return found
end


--------------------------------------------------
-- Either insert or remove the pet from the favorites table
--------------------------------------------------
function FavoritePet.SetFavorite(toggle, id)
	if toggle then
		table.insert(FavoritePet.CSV.favoritePetsList, id)
	else
		for key, value in ipairs(FavoritePet.CSV.favoritePetsList) do
			if value == id then
				table.remove(FavoritePet.CSV.favoritePetsList, key)
				break
			end
		end
	end
	local ac, acv, act, fc, fcv, fct = FavoritePet.SortPets()
	Available:UpdateChoices(acv, ac, act)
	Favorites:UpdateChoices(fcv, fc, fct)
end


--------------------------------------------------
-- SlashCommand Debug - various debug and development information triggered by the slash command
--------------------------------------------------
function FavoritePet.PingDebug()
	FavoritePet.Populate()
	d(FavoritePet.GSV.unlockedPetsList)
	d(FavoritePet.CSV.favoritePetsList)
	FavoritePet.SwitchPet()
end


-- ***** Main *****


--------------------------------------------------
-- Check to see if this addon is the one loaded
--------------------------------------------------
function FavoritePet.OnAddOnLoaded(event, addonName)
	if addonName == FavoritePet.name then
		if FavoritePet.debug then SLASH_COMMANDS[FavoritePet.slashCommand] = FavoritePet.PingDebug end
		FavoritePet.Initialize()
		
		--------------------------------------------------
		-- Was getting an error message from LAM saying the settings panel was trying to load before the
		-- rest of the addon loaded.  Wrapping it in a function seems to have fixed it.
		--------------------------------------------------
		FavoritePet.InitializeSettingsMenu()
		EVENT_MANAGER:UnregisterForEvent(FavoritePet.name, EVENT_ADD_ON_LOADED)
	end
end


EVENT_MANAGER:RegisterForEvent(FavoritePet.name, EVENT_ADD_ON_LOADED, FavoritePet.OnAddOnLoaded)


--------------------------------------------------
-- ***** Settings *****
--------------------------------------------------

--------------------------------------------------
-- Sort through the unlocked pets and return menu settings information
-- divided into available pets and favorite pets to help dynamically
-- generate the new dropdown lists
--------------------------------------------------
function FavoritePet.SortPets()
	local availableChoices = {}
	local availableChoicesValues = {}
	local availableChoicesTooltips = {}
	local favoriteChoices = {}
	local favoriteChoicesValues = {}
	local favoriteChoicesTooltips = {}
	
	for key, value in ipairs(FavoritePet.GSV.unlockedPetsList) do
		local found = FavoritePet.CheckIfFavorite(value[1])
		if found then
			table.insert(favoriteChoices, value[1])
			table.insert(favoriteChoicesValues, value[2])
			table.insert(favoriteChoicesTooltips, "|t400%:400%:"..value[3].."|t")
		else
			table.insert(availableChoices, value[1])
			table.insert(availableChoicesValues, value[2])
			table.insert(availableChoicesTooltips, "|t400%:400%:"..value[3].."|t")
		end	
	end
	return availableChoices, availableChoicesValues, availableChoicesTooltips, favoriteChoices, favoriteChoicesValues, favoriteChoicesTooltips
end


--------------------------------------------------
-- Generate the settings menu using LibAddonMenu2
--------------------------------------------------
function FavoritePet.InitializeSettingsMenu()
	if FavoritePet.debug then d("Settings Menu") end


	local LAM = LibAddonMenu2
	local panelName = "FavoritePetSettingsPanel"
		 
	local panelData = {
		type = "panel",
		name = "Favorite Pet",
		displayName = "|c00E600Favorite Pet|r",
		author = "|c787878ShadowMau|r",
		registerForRefresh = true,
		website = FavoritePet.website,
		version = FavoritePet.version
	}
	
	local optionsData = {}
	
	--------------------------------------------------
	-- Change pet when I exit combat
	--------------------------------------------------
	temptable = {
		type = "checkbox",
		name = "Change pet whenever I exit combat: ",
		getFunc = function() return FavoritePet.CSV.switchOnCombat end,
		setFunc = function(value) FavoritePet.SetCombat(value) end
	}
	table.insert(optionsData, temptable)
	
	--------------------------------------------------
	-- Change pet when the subzone changes
	--------------------------------------------------
	temptable = {
		type = "checkbox",
		name = "Change pet when the subzone changes: ",
		getFunc = function() return FavoritePet.CSV.switchOnZone end,
		setFunc = function(value) FavoritePet.SetZone(value) end
	}
	table.insert(optionsData, temptable)
	
	--------------------------------------------------
	-- Change pet when I mount
	--------------------------------------------------
	temptable = {
		type = "checkbox",
		name = "Change pet when I mount: ",
		getFunc = function() return FavoritePet.CSV.switchOnMount end,
		setFunc = function(value) FavoritePet.SetMounted(value) end
	}
	table.insert(optionsData, temptable)
	
	--------------------------------------------------
	-- Change pet when I dismount
	--------------------------------------------------
	temptable = {
		type = "checkbox",
		name = "Change pet when I dismount: ",
		getFunc = function() return FavoritePet.CSV.switchOnDismount end,
		setFunc = function(value) FavoritePet.SetDismounted(value) end
	}
	table.insert(optionsData, temptable)
	
	
	--------------------------------------------------
	-- A dividing line
	--------------------------------------------------
	local tempDivider = {
		type = "divider",
		width = "full",
		height = 10, -- optional
		alpha = 0.5 -- optional
	}
	table.insert(optionsData, tempDivider)
	
	
	--------------------------------------------------
	-- Dynamically generate the menu options
	--------------------------------------------------
	local ac, acv, act, fc, fcv, fct = FavoritePet.SortPets()
	
	temptable = {
		type = "dropdown",
		name = "Available Pets",
		width = "half",
		scrollable = true,
		reference = "Available",
		getFunc = function() return end,
		setFunc = function(petid) FavoritePet.SetFavorite(true, petid) end,
		choices = acv,
		choicesValues = ac,
		choicesTooltips = act
	}
	table.insert(optionsData, temptable)
	
	temptable = {
		type = "dropdown",
		name = "Favorite Pets",
		width = "half",
		scrollable = "true",
		reference = "Favorites",
		getFunc = function() return end,
		setFunc = function(petid) FavoritePet.SetFavorite(false, petid) end,
		choices = fcv,
		choicesValues = fc,
		choicesTooltips = fct
	}
	table.insert(optionsData, temptable)

	local panel = LAM:RegisterAddonPanel(panelName, panelData)
	LAM:RegisterOptionControls(panelName, optionsData)
end